// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2021 Florian Fischer
#include <gtest/gtest.h>

#include <string>
#include <utility>

#include "lib/LinuxVersion.hpp"

using emper::lib::LinuxVersion;

class TestLinuxVersion {
 public:
	static auto eq(std::string v1, const std::string& v2) -> bool {
		return LinuxVersion(std::move(v1)) == v2;
	}
	static auto neq(std::string v1, const std::string& v2) -> bool {
		return LinuxVersion(std::move(v1)) != v2;
	}
	static auto lt(std::string v1, const std::string& v2) -> bool {
		return LinuxVersion(std::move(v1)) < v2;
	}
	static auto le(std::string v1, const std::string& v2) -> bool {
		return LinuxVersion(std::move(v1)) <= v2;
	}
	static auto gt(std::string v1, const std::string& v2) -> bool {
		return LinuxVersion(std::move(v1)) > v2;
	}
	static auto ge(std::string v1, const std::string& v2) -> bool {
		return LinuxVersion(std::move(v1)) >= v2;
	}
};

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, eq) {
	ASSERT_TRUE(TestLinuxVersion::eq("5.13.0", "5.13.0"));
	ASSERT_TRUE(TestLinuxVersion::eq("5.13-foo.0", "5.13.0"));
}

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, neq) {
	ASSERT_TRUE(TestLinuxVersion::neq("4.1.0", "5.13.0"));
	ASSERT_TRUE(TestLinuxVersion::neq("4.1", "5.13.0"));
	ASSERT_TRUE(TestLinuxVersion::neq("5.14.0", "5.13.0"));
	ASSERT_TRUE(TestLinuxVersion::neq("5.13.1", "5.13.2"));
	ASSERT_TRUE(TestLinuxVersion::neq("5.13-foo.1", "5.13.2"));
}

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, lt) {
	ASSERT_TRUE(TestLinuxVersion::lt("4.13", "5.14.0"));
	ASSERT_TRUE(TestLinuxVersion::lt("5.13.0", "5.14.0"));
	ASSERT_TRUE(TestLinuxVersion::lt("5.13-foo.1", "5.13.2"));
	ASSERT_TRUE(TestLinuxVersion::lt("5.13.1-foo", "5.13.2"));
}

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, le) {
	ASSERT_TRUE(TestLinuxVersion::le("4.13.0", "5.14.0"));
	ASSERT_TRUE(TestLinuxVersion::le("4.14.0", "5.14.0"));
}

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, gt) {
	ASSERT_TRUE(TestLinuxVersion::gt("5.14.0", "4.13"));
	ASSERT_TRUE(TestLinuxVersion::gt("5.14.0", "5.13.0"));
	ASSERT_TRUE(TestLinuxVersion::gt("5.13.2", "5.13-foo.1"));
	ASSERT_TRUE(TestLinuxVersion::gt("5.13.2", "5.13.1-foo"));
}

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, ge) {
	ASSERT_TRUE(TestLinuxVersion::ge("5.14.0", "4.13"));
	ASSERT_TRUE(TestLinuxVersion::ge("5.14.0", "5.14.0"));
}

// NOLINTNEXTLINE(modernize-use-trailing-return-type)
TEST(LinuxVersion, real_world) {
	ASSERT_TRUE(TestLinuxVersion::le("5.15.0-0.bpo.2-amd64", "5.15.0"));
	ASSERT_TRUE(TestLinuxVersion::ge("5.17.0", "5.15.0-0.bpo.2-amd64"));
}
